<?php namespace EscapeWork\Manager\News\Models;

use EscapeWork\Manager\News\Repositories\NewsRepository;
use EscapeWork\LaravelHelpers\Eloquent;
use DB;
use DateTime;

class News extends Eloquent implements NewsRepository
{

    /**
     * Tabela
     */
    protected $table = 'news';

    /**
     * Sluggable
     */
    protected $sluggable = true;

    /**
     * Regras de validação
     */
    public static $validationRules = array(
        'news_category_id'  => array('required'),
        'title'             => array('required'),
        'description'       => array('required'),
        'date'              => array('required'),
    );

    /**
     * Mensagens de validação
     */
    public static $validationMessages = array(
        'news_category_id.required'  => 'A categoria é obrigatória',
        'slug.required'              => 'O slug é obrigatório',
        'description.required'       => 'A descrição é obrigatória',
        'date.required'              => 'A data é obrigatória',
    );

    /**
     * Fillable fields
     */
    protected $fillable = array(
        'news_category_id',
        'language',
        'title',
        'subtitle',
        'description',
        'featured',
        'active',
        'date',
        'source',
        'created_at',
        'updated_at'
    );

    /**
     * Setando a data
     */
    public function setDateAttribute($value)
    {
        if ($date = DateTime::createFromFormat('d/m/Y', $value)) {
            $this->attributes['date'] = $date->format('Y-m-d');
        }
    }

    /**
     * Setando o autor
     */
    public function setSourceAttribute($value)
    {
        $this->attributes['source'] = $value != '' ? $value : null;
    }

    /**
     * Medias relashionship
     */
    public function medias()
    {
        return $this->morphMany('EscapeWork\Manager\Medias\Models\Media', 'medias', 'media_model', 'model_id')
                    ->orderBy('order');
    }

    /**
     * Escopo para noticias ativas
     */
    public function scopeActive($query)
    {
        return $query->where('active', '=', 1);
    }

    /**
     * Escopo para noticias ativas
     */
    public function scopeFeatured($query)
    {
        return $query->where('featured', '=', 1);
    }

    /**
     * Retornando o model atual (esta função pode ser sobreescrita)
     */
    public function getMediaModel()
    {
        return 'EscapeWork\Manager\News\Models\News';
    }

    /**
     * Recupera as ultimas 15 noticias
     */
    public function getLastNews($take = 15)
    {
        return $this->active()
                    ->orderBy('news.date', 'desc')
                    ->take($take)
                    ->get();
    }

    /**
     * Excluindo uma notícia e suas mídias
     */
    public function delete()
    {
        $this->deleteMedias();

        return parent::delete();
    }

    /**
     * Excluindo as midias
     */
    protected function deleteMedias()
    {
        foreach ($this->medias as $media) {
            $media->delete();
        }
    }
}
